@@ -10,7 +10,7 @@ The Eclipse Public License is available at
 and the Eclipse Distribution License is available at
   http://www.eclipse.org/org/documents/edl-v10.php.
 
-SPDX-License-Identifier: EPL-2.0 OR EDL-1.0
+SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
 
 Contributors:
    Roger Light - initial implementation and documentation.
@@ -661,10 +661,14 @@ int config__read(struct mosquitto__config *config, bool reload)
 		}
 		mosquitto__free(config->persistence_filepath);
 		if(config->persistence_location && strlen(config->persistence_location)){
-			len = strlen(config->persistence_location) + strlen(config->persistence_file) + 1;
+			len = strlen(config->persistence_location) + strlen(config->persistence_file) + 2;
 			config->persistence_filepath = mosquitto__malloc(len);
 			if(!config->persistence_filepath) return MOSQ_ERR_NOMEM;
-			snprintf(config->persistence_filepath, len, "%s%s", config->persistence_location, config->persistence_file);
+#ifdef WIN32
+			snprintf(config->persistence_filepath, len, "%s\\%s", config->persistence_location, config->persistence_file);
+#else
+			snprintf(config->persistence_filepath, len, "%s/%s", config->persistence_location, config->persistence_file);
+#endif
 		}else{
 			config->persistence_filepath = mosquitto__strdup(config->persistence_file);
 			if(!config->persistence_filepath) return MOSQ_ERR_NOMEM;
@@ -1413,8 +1417,21 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 							{
 								for(i=0; i<config->listener_count; i++){
 									if(config->listeners[i].port == tmp_int){
-										cur_listener = &config->listeners[i];
-										break;
+										/* Now check we have a matching bind address, if defined */
+										if(config->listeners[i].host){
+											if(token && !strcmp(config->listeners[i].host, token)){
+												/* They both have a bind address, and they match */
+												cur_listener = &config->listeners[i];
+												break;
+											}
+										}else{
+											if(token == NULL){
+												/* Neither this config nor the new config have a bind address,
+												 * so they match. */
+												cur_listener = &config->listeners[i];
+												break;
+											}
+										}
 									}
 								}
 							}
@@ -1679,6 +1696,7 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 					}
 					memory__set_limit((size_t)lim);
 				}else if(!strcmp(token, "message_size_limit")){
+					log__printf(NULL, MOSQ_LOG_NOTICE, "Note: It is recommended to replace `message_size_limit` with `max_packet_size`.");
 					if(conf__parse_int(&token, "message_size_limit", (int *)&config->message_size_limit, saveptr)) return MOSQ_ERR_INVAL;
 					if(config->message_size_limit > MQTT_MAX_PAYLOAD){
 						log__printf(NULL, MOSQ_LOG_ERR, "Error: Invalid message_size_limit value (%u).", config->message_size_limit);
@@ -1820,7 +1838,6 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 						}else if(!strcmp(token, "websockets")){
 #ifdef WITH_WEBSOCKETS
 							cur_listener->protocol = mp_websockets;
-							config->have_websockets_listener = true;
 #else
 							log__printf(NULL, MOSQ_LOG_ERR, "Error: Websockets support not available.");
 							return MOSQ_ERR_INVAL;
@@ -2043,7 +2060,7 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 						token = strtok_r(NULL, " ", &saveptr);
 						if(token){
 							if (token[0] == '#'){
-								strtok_r(NULL, "", &saveptr);
+								(void)strtok_r(NULL, "", &saveptr);
 							}
 							qos = (uint8_t)atoi(token);
 							if(qos > 2){
@@ -2056,7 +2073,7 @@ int config__read_file_core(struct mosquitto__config *config, bool reload, struct
 								if(!strcmp(token, "\"\"") || token[0] == '#'){
 									local_prefix = NULL;
 									if (token[0] == '#'){
-										strtok_r(NULL, "", &saveptr);
+										(void)strtok_r(NULL, "", &saveptr);
 									}
 								}else{
 									local_prefix = token;
@@ -2271,7 +2288,7 @@ static int config__check(struct mosquitto__config *config)
 				if(!config->listeners[i].security_options.auto_id_prefix){
 					return MOSQ_ERR_NOMEM;
 				}
-				config->listeners[i].security_options.auto_id_prefix_len = strlen("auto-");
+				config->listeners[i].security_options.auto_id_prefix_len = (uint16_t)strlen("auto-");
 			}
 		}
 	}else{
@@ -2280,7 +2297,7 @@ static int config__check(struct mosquitto__config *config)
 			if(!config->security_options.auto_id_prefix){
 				return MOSQ_ERR_NOMEM;
 			}
-			config->security_options.auto_id_prefix_len = strlen("auto-");
+			config->security_options.auto_id_prefix_len = (uint16_t)strlen("auto-");
 		}
 	}
 
