@@ -10,7 +10,7 @@ The Eclipse Public License is available at
 and the Eclipse Distribution License is available at
   http://www.eclipse.org/org/documents/edl-v10.php.
 
-SPDX-License-Identifier: EPL-2.0 OR EDL-1.0
+SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
 
 Contributors:
    Roger Light - initial implementation and documentation.
@@ -54,6 +54,7 @@ Contributors:
 
 #include "mosquitto_broker_internal.h"
 #include "memory_mosq.h"
+#include "mux.h"
 #include "packet_mosq.h"
 #include "send_mosq.h"
 #include "sys_tree.h"
@@ -115,8 +116,8 @@ int mux_epoll__add_out(struct mosquitto *context)
 {
 	struct epoll_event ev;
 
-	memset(&ev, 0, sizeof(struct epoll_event));
 	if(!(context->events & EPOLLOUT)) {
+		memset(&ev, 0, sizeof(struct epoll_event));
 		ev.data.ptr = context;
 		ev.events = EPOLLIN | EPOLLOUT;
 		if(epoll_ctl(db.epollfd, EPOLL_CTL_ADD, context->sock, &ev) == -1) {
@@ -134,8 +135,8 @@ int mux_epoll__remove_out(struct mosquitto *context)
 {
 	struct epoll_event ev;
 
-	memset(&ev, 0, sizeof(struct epoll_event));
 	if(context->events & EPOLLOUT) {
+		memset(&ev, 0, sizeof(struct epoll_event));
 		ev.data.ptr = context;
 		ev.events = EPOLLIN;
 		if(epoll_ctl(db.epollfd, EPOLL_CTL_ADD, context->sock, &ev) == -1) {
