@@ -10,7 +10,7 @@ The Eclipse Public License is available at
 and the Eclipse Distribution License is available at
   http://www.eclipse.org/org/documents/edl-v10.php.
  
-SPDX-License-Identifier: EPL-2.0 OR EDL-1.0
+SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
 
 Contributors:
    Roger Light - initial implementation and documentation.
@@ -56,18 +56,18 @@ Contributors:
 #include "util_mosq.h"
 #include "mux.h"
 
-static void loop_handle_reads_writes(struct pollfd *pollfds);
+static void loop_handle_reads_writes(void);
 
 static struct pollfd *pollfds = NULL;
-static size_t pollfd_max;
+static size_t pollfd_max, pollfd_current_max;
 #ifndef WIN32
 static sigset_t my_sigblock;
 #endif
 
 int mux_poll__init(struct mosquitto__listener_sock *listensock, int listensock_count)
 {
-	int i;
-	int pollfd_index = 0;
+	size_t i;
+	size_t pollfd_index = 0;
 
 #ifndef WIN32
 	sigemptyset(&my_sigblock);
@@ -84,7 +84,7 @@ int mux_poll__init(struct mosquitto__listener_sock *listensock, int listensock_c
 	pollfd_max = (size_t)sysconf(_SC_OPEN_MAX);
 #endif
 
-	pollfds = mosquitto__malloc(sizeof(struct pollfd)*pollfd_max);
+	pollfds = mosquitto__calloc(pollfd_max, sizeof(struct pollfd));
 	if(!pollfds){
 		log__printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
 		return MOSQ_ERR_NOMEM;
@@ -94,35 +94,42 @@ int mux_poll__init(struct mosquitto__listener_sock *listensock, int listensock_c
 		pollfds[i].fd = INVALID_SOCKET;
 	}
 
-	for(i=0; i<listensock_count; i++){
+	for(i=0; i<(size_t )listensock_count; i++){
 		pollfds[pollfd_index].fd = listensock[i].sock;
 		pollfds[pollfd_index].events = POLLIN;
 		pollfds[pollfd_index].revents = 0;
 		pollfd_index++;
 	}
 
+	pollfd_current_max = pollfd_index-1;
 	return MOSQ_ERR_SUCCESS;
 }
 
 
 int mux_poll__add_out(struct mosquitto *context)
 {
-	int i;
+	size_t i;
 
-	if(context->pollfd_index != -1){
-		pollfds[context->pollfd_index].fd = context->sock;
-		pollfds[context->pollfd_index].events = POLLIN | POLLOUT;
-		pollfds[context->pollfd_index].revents = 0;
-	}else{
-		for(i=0; i<pollfd_max; i++){
-			if(pollfds[i].fd == INVALID_SOCKET){
-				pollfds[i].fd = context->sock;
-				pollfds[i].events = POLLIN | POLLOUT;
-				pollfds[i].revents = 0;
-				context->pollfd_index = i;
-				break;
+	if(!(context->events & POLLOUT)) {
+		if(context->pollfd_index != -1){
+			pollfds[context->pollfd_index].fd = context->sock;
+			pollfds[context->pollfd_index].events = POLLIN | POLLOUT;
+			pollfds[context->pollfd_index].revents = 0;
+		}else{
+			for(i=0; i<pollfd_max; i++){
+				if(pollfds[i].fd == INVALID_SOCKET){
+					pollfds[i].fd = context->sock;
+					pollfds[i].events = POLLIN | POLLOUT;
+					pollfds[i].revents = 0;
+					context->pollfd_index = (int )i;
+					if(i > pollfd_current_max){
+						pollfd_current_max = i;
+					}
+					break;
+				}
 			}
 		}
+		context->events = POLLIN | POLLOUT;
 	}
 
 	return MOSQ_ERR_SUCCESS;
@@ -131,17 +138,21 @@ int mux_poll__add_out(struct mosquitto *context)
 
 int mux_poll__remove_out(struct mosquitto *context)
 {
-	return mux_poll__add_in(context);
+	if(context->events & POLLOUT) {
+		return mux_poll__add_in(context);
+	}else{
+		return MOSQ_ERR_SUCCESS;
+	}
 }
 
 
 int mux_poll__add_in(struct mosquitto *context)
 {
-	int i;
+	size_t i;
 
 	if(context->pollfd_index != -1){
 		pollfds[context->pollfd_index].fd = context->sock;
-		pollfds[context->pollfd_index].events = POLLIN | POLLPRI;
+		pollfds[context->pollfd_index].events = POLLIN;
 		pollfds[context->pollfd_index].revents = 0;
 	}else{
 		for(i=0; i<pollfd_max; i++){
@@ -149,22 +160,39 @@ int mux_poll__add_in(struct mosquitto *context)
 				pollfds[i].fd = context->sock;
 				pollfds[i].events = POLLIN;
 				pollfds[i].revents = 0;
-				context->pollfd_index = i;
+				context->pollfd_index = (int )i;
+				if(i > pollfd_current_max){
+					pollfd_current_max = i;
+				}
 				break;
 			}
 		}
 	}
+	context->events = POLLIN;
 
 	return MOSQ_ERR_SUCCESS;
 }
 
 int mux_poll__delete(struct mosquitto *context)
 {
+	size_t pollfd_index;
+
 	if(context->pollfd_index != -1){
 		pollfds[context->pollfd_index].fd = INVALID_SOCKET;
 		pollfds[context->pollfd_index].events = 0;
 		pollfds[context->pollfd_index].revents = 0;
+		pollfd_index = (size_t )context->pollfd_index;
 		context->pollfd_index = -1;
+
+		/* If this is the highest index, reduce the current max until we find
+		 * the next highest in use index. */
+		while(pollfd_index == pollfd_current_max
+				&& pollfd_index > 0
+				&& pollfds[pollfd_index].fd == INVALID_SOCKET){
+
+			pollfd_index--;
+			pollfd_current_max--;
+		}
 	}
 
 	return MOSQ_ERR_SUCCESS;
@@ -184,10 +212,10 @@ int mux_poll__handle(struct mosquitto__listener_sock *listensock, int listensock
 
 #ifndef WIN32
 	sigprocmask(SIG_SETMASK, &my_sigblock, &origsig);
-	fdcount = poll(pollfds, pollfd_max, 100);
+	fdcount = poll(pollfds, pollfd_current_max+1, 100);
 	sigprocmask(SIG_SETMASK, &origsig, NULL);
 #else
-	fdcount = WSAPoll(pollfds, pollfd_max, 100);
+	fdcount = WSAPoll(pollfds, pollfd_current_max+1, 100);
 #endif
 
 	db.now_s = mosquitto_time();
@@ -207,10 +235,10 @@ int mux_poll__handle(struct mosquitto__listener_sock *listensock, int listensock
 			log__printf(NULL, MOSQ_LOG_ERR, "Error in poll: %s.", strerror(errno));
 		}
 	}else{
-		loop_handle_reads_writes(pollfds);
+		loop_handle_reads_writes();
 
 		for(i=0; i<listensock_count; i++){
-			if(pollfds[i].revents & (POLLIN | POLLPRI)){
+			if(pollfds[i].revents & POLLIN){
 #ifdef WITH_WEBSOCKETS
 				if(listensock[i].listener->ws_context){
 					/* Nothing needs to happen here, because we always call lws_service in the loop.
@@ -239,7 +267,7 @@ int mux_poll__cleanup(void)
 }
 
 
-static void loop_handle_reads_writes(struct pollfd *pollfds)
+static void loop_handle_reads_writes(void)
 {
 	struct mosquitto *context, *ctxt_tmp;
 	int err;
