@@ -10,7 +10,7 @@ The Eclipse Public License is available at
 and the Eclipse Distribution License is available at
   http://www.eclipse.org/org/documents/edl-v10.php.
 
-SPDX-License-Identifier: EPL-2.0 OR EDL-1.0
+SPDX-License-Identifier: EPL-2.0 OR BSD-3-Clause
 
 Contributors:
    Roger Light - initial implementation and documentation.
@@ -188,9 +188,6 @@ static int callback_mqtt(
 				if(mosq->sock != INVALID_SOCKET){
 					HASH_DELETE(hh_sock, db.contexts_by_sock, mosq);
 					mosq->sock = INVALID_SOCKET;
-#ifndef WITH_EPOLL
-					mosq->pollfd_index = -1;
-#endif
 					mux__delete(mosq);
 				}
 				mosq->wsi = NULL;
@@ -335,7 +332,7 @@ static int callback_mqtt(
 						mosq->in_packet.remaining_length += (byte & 127) * mosq->in_packet.remaining_mult;
 						mosq->in_packet.remaining_mult *= 128;
 					}while((byte & 128) != 0);
-					mosq->in_packet.remaining_count = (int8_t)(mosq->in_packet.remaining_count -1);
+					mosq->in_packet.remaining_count = (int8_t)(mosq->in_packet.remaining_count * -1);
 
 					if(mosq->in_packet.remaining_length > 0){
 						mosq->in_packet.payload = mosquitto__malloc(mosq->in_packet.remaining_length*sizeof(uint8_t));
@@ -608,24 +605,43 @@ static int callback_http(
 			break;
 
 		case LWS_CALLBACK_ADD_POLL_FD:
-		case LWS_CALLBACK_DEL_POLL_FD:
-		case LWS_CALLBACK_CHANGE_MODE_POLL_FD:
 			HASH_FIND(hh_sock, db.contexts_by_sock, &pollargs->fd, sizeof(pollargs->fd), mosq);
 			if(mosq){
-				if(pollargs->events & POLLOUT){
+				if(pollargs->events & LWS_POLLOUT){
 					mux__add_out(mosq);
 					mosq->ws_want_write = true;
 				}else{
 					mux__remove_out(mosq);
 				}
 			}else{
-				if(reason == LWS_CALLBACK_ADD_POLL_FD && (pollargs->events & POLLIN)){
+				if(pollargs->events & POLLIN){
 					/* Assume this is a new listener */
 					listeners__add_websockets(lws_get_context(wsi), pollargs->fd);
 				}
 			}
 			break;
 
+		case LWS_CALLBACK_DEL_POLL_FD:
+			HASH_FIND(hh_sock, db.contexts_by_sock, &pollargs->fd, sizeof(pollargs->fd), mosq);
+			if(mosq){
+				mux__delete(mosq);
+			}
+			break;
+
+		case LWS_CALLBACK_CHANGE_MODE_POLL_FD:
+			HASH_FIND(hh_sock, db.contexts_by_sock, &pollargs->fd, sizeof(pollargs->fd), mosq);
+			if(mosq){
+				if(pollargs->events & LWS_POLLHUP){
+					return 1;
+				}else if(pollargs->events & LWS_POLLOUT){
+					mux__add_out(mosq);
+					mosq->ws_want_write = true;
+				}else{
+					mux__remove_out(mosq);
+				}
+			}
+			break;
+
 #ifdef WITH_TLS
 		case LWS_CALLBACK_OPENSSL_PERFORM_CLIENT_CERT_VERIFICATION:
 			if(!len || (SSL_get_verify_result((SSL*)in) != X509_V_OK)){
