@@ -27,6 +27,7 @@ tests = [
     (1, './02-shared-qos0-v5.py'),
     (1, './02-subhier-crash.py'),
     (1, './02-subpub-qos0-long-topic.py'),
+    (1, './02-subpub-qos0-oversize-payload.py'),
     (1, './02-subpub-qos0-retain-as-publish.py'),
     (1, './02-subpub-qos0-send-retain.py'),
     (1, './02-subpub-qos0-subscription-id.py'),
@@ -39,6 +40,7 @@ tests = [
     (1, './02-subpub-qos1-message-expiry-will.py'),
     (1, './02-subpub-qos1-message-expiry.py'),
     (1, './02-subpub-qos1-nolocal.py'),
+    (1, './02-subpub-qos1-oversize-payload.py'),
     (1, './02-subpub-qos1.py'),
     (1, './02-subpub-qos2-1322.py'),
     (1, './02-subpub-qos2-bad-puback-1.py'),
@@ -120,6 +122,7 @@ tests = [
     (1, './07-will-no-flag.py'),
     (1, './07-will-null-topic.py'),
     (1, './07-will-null.py'),
+    (1, './07-will-oversize-payload.py'),
     (1, './07-will-properties.py'),
     (1, './07-will-qos0.py'),
     (1, './07-will-reconnect-1273.py'),
